//
//  TiledView.m
//  Field
//
//  Created by Michael Heyeck on 7/19/10.
//  Copyright 2010 Fair Oaks Labs, Inc. All rights reserved.
//

#import "Path.h"
#import "PathGroup.h"
#import "TiledView.h"
#import <QuartzCore/QuartzCore.h>


@interface TiledView ()

- (NSArray*)pathGroupsForTile:(CGPoint)tile;

@end


@implementation TiledView

+ (Class)layerClass
{
	return [CATiledLayer class];
}


- (id)initWithFrame:(CGRect)frame
{
    if (self = [super initWithFrame:frame])
	{
        // Initialization code
    }
    return self;
}


/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/


- (void)dealloc
{
    [super dealloc];
}

#pragma mark Tiled layer delegate methods

- (void)drawLayer:(CALayer*)layer inContext:(CGContextRef)context
{
	// Fetch clip box in *world* space; context's CTM is preconfigured for world space->tile pixel space transform
	CGRect box = CGContextGetClipBoundingBox(context);
	
	// Calculate tile index
	CGSize tileSize = [(CATiledLayer*)layer tileSize];
	CGRect tbox = CGRectApplyAffineTransform(CGRectMake(0, 0, tileSize.width, tileSize.height), 
											 CGAffineTransformInvert(CGContextGetCTM(context)));
	CGFloat x = box.origin.x / tbox.size.width;
	CGFloat y = box.origin.y / tbox.size.height;
	CGPoint tile = CGPointMake(x, y);
	
	// Calculate LOD
	CGFloat scale = tbox.size.width/tileSize.width;
	
	// Clear background
	CGContextSetFillColorWithColor(context, [[UIColor grayColor] CGColor]);
	CGContextFillRect(context, box);
	
	// Rendering the paths
	NSArray* pathGroups = [self pathGroupsForTile:tile];
	for (PathGroup* pg in pathGroups)
	{
		CGContextSaveGState(context);
		
		CGContextConcatCTM(context, pg.modelTransform);
		
		for (Path* p in pg.paths)
		{
			[p renderToContext:context];
		}
		
		CGContextRestoreGState(context);
	}
	
	// Render label (Setup)
	UIFont* font = [UIFont fontWithName:@"CourierNewPS-BoldMT" size:16];
	CGContextSelectFont(context, [[font fontName] cStringUsingEncoding:NSASCIIStringEncoding], [font pointSize], kCGEncodingMacRoman);
	CGContextSetTextDrawingMode(context, kCGTextFill);
	CGContextSetTextMatrix(context, CGAffineTransformMakeScale(scale, -scale));
	CGContextSetFillColorWithColor(context, [[UIColor greenColor] CGColor]);
	
	// Draw label
	NSString* s = [NSString stringWithFormat:@"(%.1f, %.1f)",x,y];
	CGContextShowTextAtPoint(context,
							 box.origin.x,
							 box.origin.y + [font pointSize]*scale,
							 [s cStringUsingEncoding:NSMacOSRomanStringEncoding],
							 [s lengthOfBytesUsingEncoding:NSMacOSRomanStringEncoding]);
}

#pragma mark Extension methods

- (NSArray*)pathGroupsForTile:(CGPoint)tile
{
	// Just for testing, mind you!
	Path*				path;
	PathGroup*			pg;
	CGMutablePathRef	p;
	NSMutableArray*		a = [NSMutableArray arrayWithCapacity:2];
	
	// Group 1
	pg = [[PathGroup new] autorelease];
	pg.modelTransform =  CGAffineTransformMakeTranslation(320, 460);
	[a addObject:pg];
	// Group 1, Path 1
	path = [[Path new] autorelease];
	p = CGPathCreateMutable();
	CGPathAddEllipseInRect(p, NULL, CGRectMake(-95, -95, 190, 190));
	path.path = p;
	path.strokeWidth = 10;
	CGPathRelease(p);
	[pg.paths addObject:path];
	// Group 1, Path 2
	path = [[Path new] autorelease];
	p = CGPathCreateMutable();
	CGPathAddRect(p, NULL, CGRectMake(-69.5, -51.5, 139, 103));
	path.path = p;
	path.strokeWidth = 5;
	CGPathRelease(p);
	[pg.paths addObject:path];
	// Group 2
	pg = [[PathGroup new] autorelease];
	pg.modelTransform =  CGAffineTransformMakeTranslation(480, 230);
	[a addObject:pg];
	// Group 2, Path 1
	path = [[Path new] autorelease];
	p = CGPathCreateMutable();
	CGPathAddRect(p, NULL, CGRectMake(-25, -25, 50, 50));
	path.path = p;
	path.strokeWidth = 2;
	CGPathRelease(p);
	[pg.paths addObject:path];
	
	// For now, all tiles get the same renderlist
	return a;
}

@end
